MessageClass

Enum MessageClass 

pub enum MessageClass {
    Request,
    Indication,
    Success,
    Error,
}
The class of a Message.

There are four classes of Messages within the STUN protocol:

  • Request indicates that a request is being made and a response is expected.
  • An Indication is a fire and forget Message where no response is required or expected.
  • Success indicates that a Request was successfully handled and the
  • Error class indicates that an error was produced.

Variants



Request

A request that is expecting a response of either Success, or Error.



Indication

A request that does not expect a response.



Success

A success response to a previous Request.



Error

An error response to a previous Request.

Implementations

Source

impl MessageClass

pub fn is_response(self) -> bool

Returns whether this MessageClass is of a response type. i.e. is either MessageClass::Success or MessageClass::Error.