#[non_exhaustive]pub enum StunParseError {
NotStun,
Truncated {
expected: usize,
actual: usize,
},
TooLarge {
expected: usize,
actual: usize,
},
MissingAttribute(AttributeType),
AttributeAfterIntegrity(AttributeType),
AttributeAfterFingerprint(AttributeType),
FingerprintMismatch,
DataMismatch,
InvalidAttributeData,
WrongAttributeImplementation,
}Expand description
Possible errors when parsing a STUN message.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
NotStun
Not a STUN message.
Truncated
The message has been truncated
TooLarge
The message has been truncated
MissingAttribute(AttributeType)
An attribute was not found in the message
AttributeAfterIntegrity(AttributeType)
An attribute was found after the message integrity attribute
AttributeAfterFingerprint(AttributeType)
An attribute was found after the message integrity attribute
FingerprintMismatch
Fingerprint does not match the data.
DataMismatch
The provided data does not match the message
InvalidAttributeData
The attribute contains invalid data
WrongAttributeImplementation
The attribute does not parse this data